// ****************************
// ***   us flag - hires    ***
// ***     rle-encoded      ***
// ***      9-nov-2023      ***
// ***  by kevin krausnick  ***
// ****************************

// Assembler: Kick

.const vic_ctrl_reg_1      = $d011 // 53265
.const vic_mem_ctrl_reg    = $d018 // 53272
.const border_color        = $d020 // 53280
.const background_color    = $d021 // 53281
.const cia_ii              = $dd00 // 56576
.const initscr             = $e544 // 58692
.const getin               = $ffe4 // 65508
.const black               = 0
.const white               = 1

.const imgloc   = $4000 + $2000
.const colorloc = $4000 + $0400
.const zp_ptr1  = $fb
.const zp_ptr2  = $fe

* = $0801 // start of basic

// basic loader
.byte $28,$08,$00,$00,$9e,$32,$30,$39,$30,$3a,$8f,$20
.text "9-NOV-2023 BY K. KRAUSNICK"
.byte 0,0,0

init:
  lda border_color
  sta saved_values
  lda #black
  sta border_color
  lda background_color
  sta saved_values+1
  lda #white
  sta background_color
  jsr initscr
  lda vic_ctrl_reg_1
  and #$ef    // blank screen while rendering
  ora #$20    // enter bit map mode
  sta vic_ctrl_reg_1
  lda #$02    // video bank 1
  sta cia_ii
  lda vic_mem_ctrl_reg
  ora #$08    // video base address ($2000 offset from $4000 bank)
  sta vic_mem_ctrl_reg

// copy color data (1000 bytes)
  lda #<palette
  sta zp_ptr1
  lda #>palette
  sta zp_ptr1+1
  lda #<colorloc
  sta zp_ptr2
  lda #>colorloc
  sta zp_ptr2+1
  jsr rle_decompress

// copy image data (8000 bytes)
  lda #<bitmap
  sta zp_ptr1
  lda #>bitmap
  sta zp_ptr1+1
  lda #<imgloc
  sta zp_ptr2
  lda #>imgloc
  sta zp_ptr2+1
  jsr rle_decompress

// reveal bitmap and wait for keypress
  lda vic_ctrl_reg_1
  ora #$10    // unblank screen
  sta vic_ctrl_reg_1
!: jsr getin
  beq !-

// cleanup and exit
  lda #$03    // video bank 0
  sta cia_ii
  lda vic_ctrl_reg_1
  and #$df    // switch back to text mode
  sta vic_ctrl_reg_1
  lda vic_mem_ctrl_reg
  and #$f7    // restore video base address
  sta vic_mem_ctrl_reg
  lda saved_values
  sta border_color
  lda saved_values+1
  sta background_color
  rts

// decompress image or color data while copying
rle_decompress:
!: ldy #0
  lda (zp_ptr1),y
  beq !++
  tax
  pha
  iny
  lda (zp_ptr1),y
  dey
!: sta (zp_ptr2),y
  iny
  dex
  bne !-
  pla
  clc
  adc zp_ptr2
  sta zp_ptr2
  lda zp_ptr2+1
  adc #0
  sta zp_ptr2+1
  lda zp_ptr1
  clc
  adc #2
  sta zp_ptr1
  lda zp_ptr1+1
  adc #0
  sta zp_ptr1+1
  jmp !--
!: rts

saved_values: .fill 2,0

// color data (rle-compressed)
palette:
.byte $10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12
.byte $10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$18,$12,$10,$16,$ff,$12,$d1,$12,$00,$00

// image data (rle-compressed)
bitmap:
.byte $0e,$00,$01,$04,$01,$0c,$16,$00,$01,$40,$01,$c0,$0e,$00,$01,$04,$01,$06,$16,$00,$01,$40,$01,$60,$0e,$00,$01,$02,$01,$06,$16,$00
.byte $01,$20,$01,$60,$ca,$00,$01,$01,$05,$00,$01,$0e,$01,$3f,$01,$ff,$01,$7f,$03,$3f,$01,$21,$02,$00,$01,$e0,$01,$80,$03,$00,$01,$80
.byte $01,$00,$01,$03,$01,$0f,$01,$07,$03,$03,$01,$02,$01,$e0,$01,$f8,$01,$fe,$01,$f8,$02,$f0,$01,$b8,$01,$08,$08,$00,$01,$0e,$01,$3f
.byte $01,$ff,$01,$3f,$01,$1f,$01,$3f,$01,$3b,$01,$20,$01,$00,$01,$80,$01,$e0,$05,$80,$01,$00,$01,$03,$01,$0f,$01,$03,$03,$01,$01,$03
.byte $01,$e0,$01,$f8,$01,$ff,$01,$fc,$03,$f8,$01,$08,$08,$00,$01,$07,$01,$1f,$01,$ff,$01,$3f,$03,$1f,$01,$10,$01,$00,$01,$80,$01,$f0
.byte $01,$c0,$03,$80,$01,$c0,$01,$00,$01,$01,$01,$07,$01,$03,$04,$01,$01,$70,$01,$fc,$01,$ff,$01,$fc,$02,$f8,$01,$dc,$01,$04,$da,$00
.byte $02,$01,$01,$07,$01,$3f,$01,$0f,$01,$07,$01,$00,$02,$80,$01,$c0,$01,$e0,$01,$fc,$01,$f0,$01,$e0,$05,$00,$01,$01,$03,$00,$01,$08
.byte $01,$18,$01,$1c,$01,$7f,$02,$ff,$01,$7e,$05,$00,$01,$c0,$05,$00,$01,$01,$01,$07,$01,$1f,$01,$07,$01,$03,$01,$00,$01,$80,$02,$c0
.byte $01,$f0,$01,$fc,$02,$f0,$05,$00,$01,$01,$03,$00,$01,$08,$01,$0c,$01,$1c,$01,$7f,$01,$ff,$01,$7f,$01,$3f,$05,$00,$01,$e0,$01,$80
.byte $05,$00,$01,$03,$01,$1f,$01,$07,$01,$03,$01,$00,$01,$40,$01,$c0,$01,$e0,$01,$f0,$01,$fe,$01,$f8,$01,$f0,$10,$00,$c0,$ff,$07,$00
.byte $01,$01,$03,$00,$01,$04,$01,$0c,$01,$0e,$01,$3f,$01,$ff,$02,$07,$01,$04,$04,$00,$03,$e0,$01,$30,$03,$00,$01,$03,$01,$0f,$03,$00
.byte $01,$40,$01,$c0,$01,$e0,$01,$f8,$01,$fe,$01,$7e,$01,$77,$01,$41,$08,$00,$01,$04,$01,$06,$01,$0e,$01,$3f,$01,$ff,$02,$07,$01,$04
.byte $03,$00,$01,$80,$01,$e0,$01,$f0,$01,$70,$01,$10,$03,$00,$01,$03,$01,$0f,$03,$00,$01,$40,$01,$60,$01,$e0,$01,$f8,$01,$ff,$02,$3f
.byte $01,$61,$08,$00,$01,$02,$01,$06,$01,$07,$01,$1f,$01,$ff,$02,$03,$01,$02,$03,$00,$01,$80,$03,$f0,$01,$18,$03,$00,$01,$01,$01,$07
.byte $03,$00,$01,$20,$01,$60,$01,$70,$01,$fc,$01,$ff,$08,$00,$c0,$ff,$08,$00,$01,$7f,$03,$3f,$01,$21,$03,$00,$01,$80,$03,$00,$01,$80
.byte $02,$00,$01,$01,$01,$07,$03,$03,$01,$02,$01,$00,$02,$80,$01,$f8,$02,$f0,$01,$b8,$01,$08,$09,$00,$01,$08,$01,$18,$01,$3f,$01,$1f
.byte $01,$3f,$01,$3b,$01,$20,$03,$00,$05,$80,$03,$00,$01,$03,$03,$01,$01,$03,$01,$00,$01,$80,$01,$c0,$01,$fc,$03,$f8,$01,$08,$09,$00
.byte $01,$08,$01,$0c,$01,$3f,$03,$1f,$01,$10,$03,$00,$01,$c0,$03,$80,$01,$c0,$03,$00,$01,$03,$04,$01,$01,$00,$01,$40,$01,$c0,$01,$fc
.byte $02,$f8,$01,$dc,$01,$04,$db,$00,$01,$01,$01,$07,$01,$3f,$01,$0f,$03,$07,$01,$04,$01,$c0,$01,$e0,$01,$fc,$01,$f0,$03,$e0,$01,$30
.byte $02,$00,$01,$01,$05,$00,$01,$1c,$01,$7f,$02,$ff,$02,$7e,$01,$77,$01,$41,$02,$00,$01,$c0,$05,$00,$01,$01,$01,$07,$01,$1f,$01,$07
.byte $01,$03,$02,$07,$01,$04,$01,$c0,$01,$f0,$01,$fc,$03,$f0,$01,$70,$01,$10,$02,$00,$01,$01,$05,$00,$01,$1c,$01,$7f,$01,$ff,$01,$7f
.byte $03,$3f,$01,$61,$02,$00,$01,$e0,$01,$80,$05,$00,$01,$03,$01,$1f,$01,$07,$03,$03,$01,$02,$01,$e0,$01,$f0,$01,$fe,$01,$f8,$03,$f0
.byte $01,$18,$d5,$00,$01,$01,$03,$00,$01,$04,$01,$0c,$01,$0e,$01,$3f,$01,$ff,$01,$7f,$01,$3f,$05,$00,$01,$e0,$01,$80,$05,$00,$01,$03
.byte $01,$0f,$01,$07,$01,$03,$01,$00,$01,$40,$01,$c0,$01,$e0,$01,$f8,$01,$fe,$01,$f8,$01,$f0,$09,$00,$01,$04,$01,$06,$01,$0e,$01,$3f
.byte $01,$ff,$01,$3f,$01,$1f,$04,$00,$01,$80,$01,$e0,$02,$80,$04,$00,$01,$03,$01,$0f,$01,$03,$01,$01,$01,$00,$01,$40,$01,$60,$01,$e0
.byte $01,$f8,$01,$ff,$01,$fc,$01,$f8,$09,$00,$01,$02,$01,$06,$01,$07,$01,$1f,$01,$ff,$01,$3f,$01,$1f,$04,$00,$01,$80,$01,$f0,$01,$c0
.byte $01,$80,$04,$00,$01,$01,$01,$07,$01,$03,$01,$01,$01,$00,$01,$20,$01,$60,$01,$70,$01,$fc,$01,$ff,$01,$fc,$01,$f8,$08,$00,$c0,$ff
.byte $08,$00,$02,$3f,$01,$21,$07,$00,$01,$80,$02,$00,$02,$01,$01,$07,$02,$03,$01,$02,$01,$00,$02,$80,$01,$c0,$01,$e0,$01,$f0,$01,$b8
.byte $01,$08,$09,$00,$01,$08,$01,$18,$01,$1c,$01,$7f,$01,$3f,$01,$3b,$01,$20,$05,$00,$03,$80,$03,$00,$01,$01,$01,$07,$02,$01,$01,$03
.byte $01,$00,$01,$80,$02,$c0,$01,$f0,$02,$f8,$01,$08,$09,$00,$01,$08,$01,$0c,$01,$1c,$01,$7f,$02,$1f,$01,$10,$05,$00,$02,$80,$01,$c0
.byte $04,$00,$01,$03,$03,$01,$01,$00,$01,$40,$01,$c0,$01,$e0,$01,$f0,$01,$f8,$01,$dc,$01,$04,$0d,$00,$c0,$ff,$0f,$00,$01,$02,$01,$3f
.byte $01,$0f,$03,$07,$01,$04,$02,$00,$01,$fc,$01,$f0,$03,$e0,$01,$30,$02,$00,$01,$01,$06,$00,$01,$20,$02,$ff,$02,$7e,$01,$77,$01,$41
.byte $02,$00,$01,$c0,$06,$00,$01,$02,$01,$1f,$01,$07,$01,$03,$02,$07,$01,$04,$02,$00,$01,$fc,$03,$f0,$01,$70,$01,$10,$02,$00,$01,$01
.byte $06,$00,$01,$20,$01,$ff,$01,$7f,$03,$3f,$01,$61,$02,$00,$01,$e0,$01,$80,$05,$00,$01,$01,$01,$1f,$01,$07,$03,$03,$01,$02,$02,$00
.byte $01,$fe,$01,$f8,$03,$f0,$01,$18,$09,$00,$01,$10,$d0,$00,$01,$06,$01,$07,$01,$1f,$01,$ff,$01,$3f,$03,$1f,$02,$00,$01,$80,$01,$f0
.byte $01,$c0,$03,$80,$02,$00,$01,$01,$01,$07,$01,$03,$03,$01,$01,$60,$01,$70,$01,$fc,$01,$ff,$01,$fc,$02,$f8,$01,$dc,$08,$00,$01,$03
.byte $01,$07,$01,$1f,$01,$7f,$01,$1f,$01,$0f,$01,$1f,$01,$1d,$02,$00,$01,$c0,$01,$f0,$04,$c0,$02,$00,$01,$01,$01,$07,$01,$01,$03,$00
.byte $01,$30,$01,$70,$01,$fc,$01,$ff,$01,$fe,$03,$fc,$03,$00,$01,$80,$04,$00,$02,$03,$01,$0f,$01,$7f,$01,$1f,$03,$0f,$01,$00,$01,$80
.byte $01,$c0,$01,$f8,$01,$e0,$03,$c0,$03,$00,$01,$03,$01,$01,$03,$00,$01,$30,$01,$38,$01,$fe,$01,$ff,$01,$fe,$02,$fc,$01,$ee,$03,$00
.byte $01,$80,$cc,$00,$01,$10,$07,$00,$01,$c0,$04,$00,$01,$03,$01,$1f,$01,$07,$01,$01,$01,$00,$01,$40,$01,$c0,$01,$e0,$01,$f0,$01,$fe
.byte $01,$f8,$01,$04,$09,$00,$01,$04,$01,$0c,$01,$0e,$01,$3f,$01,$ff,$01,$7f,$01,$10,$04,$00,$01,$80,$01,$e0,$01,$80,$01,$40,$04,$00
.byte $01,$03,$01,$0f,$01,$03,$01,$01,$01,$00,$01,$40,$01,$60,$01,$e0,$01,$f8,$01,$fe,$01,$f8,$01,$84,$09,$00,$01,$04,$01,$06,$01,$0e
.byte $01,$3f,$01,$ff,$01,$3f,$01,$08,$04,$00,$01,$80,$01,$f0,$01,$c0,$01,$60,$04,$00,$01,$01,$01,$0f,$01,$03,$02,$00,$01,$20,$01,$60
.byte $01,$70,$01,$f8,$01,$ff,$01,$fc,$01,$82,$0f,$00,$c0,$ff,$0d,$00,$01,$01,$05,$03,$01,$02,$03,$00,$01,$80,$03,$f0,$01,$18,$09,$00
.byte $01,$10,$01,$30,$01,$38,$02,$3f,$01,$3b,$01,$20,$06,$00,$02,$80,$01,$00,$02,$01,$01,$03,$01,$01,$02,$03,$01,$02,$02,$00,$02,$80
.byte $02,$f8,$01,$b8,$01,$08,$09,$00,$01,$10,$01,$18,$01,$38,$03,$1f,$01,$30,$04,$00,$04,$80,$02,$00,$06,$01,$01,$00,$02,$80,$01,$c0
.byte $03,$f8,$01,$0c,$09,$00,$01,$08,$01,$18,$01,$1c,$08,$00,$c0,$ff,$08,$00,$01,$0f,$01,$7f,$01,$1f,$03,$0f,$01,$08,$01,$00,$01,$c0
.byte $01,$f8,$01,$e0,$03,$c0,$01,$60,$02,$00,$01,$03,$01,$01,$05,$00,$01,$fe,$01,$ff,$01,$fe,$02,$fc,$01,$ee,$01,$82,$02,$00,$01,$80
.byte $06,$00,$01,$0f,$01,$3f,$01,$0f,$01,$07,$01,$0f,$01,$0e,$01,$08,$01,$00,$01,$e0,$01,$f8,$04,$e0,$01,$20,$02,$00,$01,$03,$06,$00
.byte $01,$fe,$02,$ff,$03,$7e,$01,$c2,$02,$00,$01,$c0,$06,$00,$01,$07,$01,$3f,$01,$0f,$03,$07,$01,$04,$01,$00,$01,$e0,$01,$fc,$01,$f0
.byte $03,$e0,$01,$30,$02,$00,$01,$01,$06,$00,$01,$7f,$02,$ff,$02,$7e,$01,$77,$01,$41,$02,$00,$01,$c0,$ff,$00,$ff,$00,$08,$00,$ff,$ff
.byte $ff,$ff,$82,$ff,$ff,$00,$ff,$00,$82,$00,$ff,$ff,$ff,$ff,$82,$ff,$ff,$00,$ff,$00,$82,$00,$ff,$ff,$ff,$ff,$82,$ff,$ff,$00,$41,$00
.byte $00